// NOTE: We check for a range of 1 to 20 for font IDs, but we must subtract 1 to fit in the array.

// Windows stuff
#include "StdAfx.h"

// Defines / Structures / Variable stuff
#include "general.h"

// Font details
FontInfo*	fiFont;

// 2D text in a box with overriding colour
void BoxTextColour(int iFont, int iX, int iY, int iW, int iH, int iHAlign, int iVAlign, LPTSTR pString, DWORD dwColour)
{
	if (fiFont[iFont-1].iCodePage==CP_ACP)
		BoxTextColourMulti(iFont, iX, iY, iW, iH, iHAlign, iVAlign, pString, dwColour);
	else
		BoxTextColourWide(iFont, iX, iY, iW, iH, iHAlign, iVAlign, pString, dwColour);
}
void BoxTextColourMulti(int iFont, int iX, int iY, int iW, int iH, int iHAlign, int iVAlign, LPTSTR pString, DWORD dwColour)
{
	// Valid font ID
	if ((iFont < 1) || (iFont > g_MaxFonts))
		return;
	else
		iFont--;

	RECT rctBox = {iX, iY, iX + iW, iY + iH};
	DWORD dwFormat = DT_WORDBREAK | DT_EXPANDTABS;

	// Horizontal alignment
	switch (iHAlign)
	{
		case 0:
			dwFormat |= DT_LEFT;
			break;
		case 1:
			dwFormat |= DT_CENTER;
			break;
		case 2:
			dwFormat |= DT_RIGHT;
			break;
	}

	// Vertical alignment
	switch (iVAlign)
	{
		case 0:
			dwFormat |= DT_TOP;
			break;
		case 1:
			dwFormat |= DT_VCENTER;
			break;
		case 2:
			dwFormat |= DT_BOTTOM;
			break;
	}

	// Draw
	fiFont[iFont].pFont->DrawText(g_pTextSprite, pString, -1, &rctBox, dwFormat, dwColour);
}
void BoxTextColourWide(int iFont, int iX, int iY, int iW, int iH, int iHAlign, int iVAlign, LPTSTR pString, DWORD dwColour)
{
	// Valid font ID
	if ((iFont < 1) || (iFont > g_MaxFonts))
		return;
	else
		iFont--;

	RECT rctBox = {iX, iY, iX + iW, iY + iH};
	DWORD dwFormat = DT_WORDBREAK | DT_EXPANDTABS;

	// Horizontal alignment
	switch (iHAlign)
	{
		case 0:
			dwFormat |= DT_LEFT;
			break;
		case 1:
			dwFormat |= DT_CENTER;
			break;
		case 2:
			dwFormat |= DT_RIGHT;
			break;
	}

	// Vertical alignment
	switch (iVAlign)
	{
		case 0:
			dwFormat |= DT_TOP;
			break;
		case 1:
			dwFormat |= DT_VCENTER;
			break;
		case 2:
			dwFormat |= DT_BOTTOM;
			break;
	}

	// Draw
	int lenA = lstrlenA(pString);
	int lenW;
	BSTR unicodestr;

	lenW = ::MultiByteToWideChar(fiFont[iFont].iCodePage, 0, pString, lenA, 0, 0);
	unicodestr = ::SysAllocStringLen(0, lenW);
	::MultiByteToWideChar(fiFont[iFont].iCodePage, 0, pString, lenA, unicodestr, lenW);

	fiFont[iFont].pFont->DrawTextW(g_pTextSprite, unicodestr, -1, &rctBox, dwFormat, dwColour);

	::SysFreeString(unicodestr);
}

// 2D text in a box
void BoxText(int iFont, int iX, int iY, int iW, int iH, int iHAlign, int iVAlign, LPTSTR pString)
{
	BoxTextColour(iFont, iX, iY, iW, iH, iHAlign, iVAlign, pString, g_Forecolour);
}

// Even prettier 2D text than the function below (Overriding colour)

void DrawAATextColour(int iFont, int iX, int iY, int iAlign, LPTSTR pString, DWORD dwColour)
{
#ifdef EXTENDEDCHARS

	if (fiFont[iFont-1].iCodePage==CP_ACP)
		DrawAATextColourMulti(iFont, iX, iY, iAlign, pString, dwColour);
	else
		DrawAATextColourWide(iFont, iX, iY, iAlign, pString, dwColour);
}

void DrawAATextColourMulti(int iFont, int iX, int iY, int iAlign, LPTSTR pString, DWORD dwColour)
{
#endif

	// Valid font ID
	if ((iFont < 1) || (iFont > g_MaxFonts))
		return;
	else
		iFont--;

	RECT rctBox = {iX, iY, iX, 0};
	//RECT rctBox = {iX, iY, 0, 0};
	// Alignment

	switch (iAlign)
	{
		// Left
		case 0:
			fiFont[iFont].pFont->DrawText(g_pTextSprite, pString, -1, &rctBox, DT_NOCLIP, dwColour);
			break;
		// Center
		case 1:
			rctBox.left -= 5000;
			rctBox.right += 5000;
			fiFont[iFont].pFont->DrawText(g_pTextSprite, pString, -1, &rctBox, DT_NOCLIP | DT_CENTER, dwColour);
			break;
		// Right
		case 2:
			rctBox.left -= 5000;
			fiFont[iFont].pFont->DrawText(g_pTextSprite, pString, -1, &rctBox, DT_NOCLIP | DT_RIGHT, dwColour);
			break;
	}
}
#ifdef EXTENDEDCHARS
void DrawAATextColourWide(int iFont, int iX, int iY, int iAlign, LPTSTR pString, DWORD dwColour)
{
	// Valid font ID
	if ((iFont < 1) || (iFont > g_MaxFonts))
		return;
	else
		iFont--;

	RECT rctBox = {iX, iY, iX, 0};
	//RECT rctBox = {iX, iY, 0, 0};
	// Alignment
	int lenA;
	int lenW;
	BSTR unicodestr;

	lenA = lstrlenA(pString);
	lenW = ::MultiByteToWideChar(fiFont[iFont].iCodePage, 0, pString, lenA, 0, 0);
	unicodestr = ::SysAllocStringLen(0, lenW);
	::MultiByteToWideChar(fiFont[iFont].iCodePage, 0, pString, lenA, unicodestr, lenW);

	switch (iAlign)
	{
		// Left
		case 0:
			fiFont[iFont].pFont->DrawTextW(g_pTextSprite, unicodestr, -1, &rctBox, DT_NOCLIP, dwColour);
			break;
		// Center
		case 1:
			rctBox.left -= 5000;
			rctBox.right += 5000;
			fiFont[iFont].pFont->DrawTextW(g_pTextSprite, unicodestr, -1, &rctBox, DT_NOCLIP | DT_CENTER, dwColour);
			break;
		// Right
		case 2:
			rctBox.left -= 5000;
			fiFont[iFont].pFont->DrawTextW(g_pTextSprite, unicodestr, -1, &rctBox, DT_NOCLIP | DT_RIGHT, dwColour);
			break;
	}
	::SysFreeString(unicodestr);

}
#endif
// Pretty 2D text
void DrawAAText(int iFont, int iX, int iY, int iAlign, LPTSTR pString)
{
	DrawAATextColour(iFont, iX, iY, iAlign, pString, g_Forecolour);
}

// Erm...Start Text
void StartText(void)
{

	g_pViewPort.X = 0;
    g_pViewPort.Y = 0;
    g_pViewPort.Width = g_pGlob->dwWindowWidth;
    g_pViewPort.Height = g_pGlob->dwWindowHeight;
	// TODO: Should these be CameraNear/Far?
    g_pViewPort.MinZ = 0.0f;
    g_pViewPort.MaxZ = 1.0f;
	g_pD3DDevice->SetViewport(&g_pViewPort);


	g_pTextSprite->Begin(D3DXSPRITE_ALPHABLEND | D3DXSPRITE_SORT_DEPTH_BACKTOFRONT);

	//g_pD3DDevice->SetRenderState(D3DRS_SRCBLEND, D3DBLEND_ONE);
	//g_pD3DDevice->SetRenderState(D3DRS_DESTBLEND, D3DBLEND_ZERO);

	//g_pD3DDevice->SetRenderState(D3DRS_ALPHABLENDENABLE, TRUE); 
	//g_pD3DDevice->SetRenderState(D3DRS_ALPHAFUNC, D3DCMP_GREATEREQUAL); 
	//g_pD3DDevice->SetRenderState(D3DRS_ALPHAREF, (DWORD)100); 
	//g_pD3DDevice->SetRenderState(D3DRS_ALPHATESTENABLE, TRUE); 
}

// Again erm...End Text
void EndText(void)
{
	g_pD3DDevice->SetRenderState(D3DRS_FILLMODE, D3DFILL_SOLID);
	g_pTextSprite->End();
}

// Initialize a font
void SetFont(int iFont, LPTSTR pFont, int iSize, int iBold, int iItalic, int iMeasurement)
{
	SetFontExt(iFont, pFont, iSize, iBold, iItalic, iMeasurement, DEFAULT_CHARSET);
}

void SetFontExt(int iFont, LPTSTR pFont, int iSize, int iBold, int iItalic, int iMeasurement, DWORD dwCharSet)
{
	// Valid font ID
	if ((iFont < 1) || (iFont > g_MaxFonts))
		return;
	else
		iFont--;

	// Add details to font arrays
	fiFont[iFont].sFontname = pFont;
	fiFont[iFont].bBold = (iBold != 0);
	fiFont[iFont].bItalic = (iItalic != 0);
	fiFont[iFont].dwCharSet=dwCharSet;

	iSize=-iSize;
	if (iMeasurement != 0)
	{
		// Calculate font size in pixels
		HDC hDC = GetDC(NULL);
		iSize = iSize * GetDeviceCaps(hDC, LOGPIXELSY) / 72;
		ReleaseDC(NULL, hDC);
	}
	fiFont[iFont].iHeight = iSize;

	HRESULT HR = D3DXCreateFont(g_pD3DDevice, iSize, 0, ((iBold != 0) ? FW_BOLD : FW_NORMAL), 1, ((iItalic != 0) ? true : false), dwCharSet, OUT_DEFAULT_PRECIS, ANTIALIASED_QUALITY, DEFAULT_PITCH || FF_DONTCARE, pFont, &fiFont[iFont].pFont);

	switch (dwCharSet)
	{
		case DEFAULT_CHARSET:
			fiFont[iFont].iCodePage=CP_ACP;
			break;
		case SYMBOL_CHARSET:
			fiFont[iFont].iCodePage=CP_SYMBOL;
			break;
		case SHIFTJIS_CHARSET:
			fiFont[iFont].iCodePage=932;
			break;
		case HANGEUL_CHARSET:
			fiFont[iFont].iCodePage=949;
			break;
		case GB2312_CHARSET:
			fiFont[iFont].iCodePage=936;
			break;
		case CHINESEBIG5_CHARSET:
			fiFont[iFont].iCodePage=950;
			break;
		case OEM_CHARSET:
			fiFont[iFont].iCodePage=0;
			break;
		case JOHAB_CHARSET:
			fiFont[iFont].iCodePage=1361;
			break;
		case HEBREW_CHARSET:
			fiFont[iFont].iCodePage=1255;
			break;
		case ARABIC_CHARSET:
			fiFont[iFont].iCodePage=1256;
			break;
		case GREEK_CHARSET:
			fiFont[iFont].iCodePage=1253;
			break;
		case TURKISH_CHARSET:
			fiFont[iFont].iCodePage=1254;
			break;
		case VIETNAMESE_CHARSET:
			fiFont[iFont].iCodePage=1258;
			break;
		case THAI_CHARSET:
			fiFont[iFont].iCodePage=874;
			break;
		case EASTEUROPE_CHARSET:
			fiFont[iFont].iCodePage=1250;
			break;
		case RUSSIAN_CHARSET:
			fiFont[iFont].iCodePage=1251;
			break;
		case MAC_CHARSET:
			fiFont[iFont].iCodePage=CP_ACP;
			break;
		case BALTIC_CHARSET:
			fiFont[iFont].iCodePage=1257;
			break;
	}
	//fiFont[iFont].pFont->PreloadGlyphs(0, 255);
}

// These commands really need better names...I believe this returns the text width...but who knows for sure
int GetTextWidth(int iFont, LPTSTR pString)
{
	// Valid font ID
	if ((iFont < 1) || (iFont > g_MaxFonts))
		return 0;
	else
		iFont--;

	RECT rctBounding;

	fiFont[iFont].pFont->DrawText(g_pTextSprite, pString, -1, &rctBounding, DT_CALCRECT, 0);

	return (rctBounding.right - rctBounding.left);
}

// Another dodgy name
int GetTextHeight(int iFont, LPTSTR pString)
{
	// Valid font ID
	if ((iFont < 1) || (iFont > g_MaxFonts))
		return 0;
	else
		iFont--;

	RECT rctBounding;

	fiFont[iFont].pFont->DrawText(g_pTextSprite, pString, -1, &rctBounding, DT_CALCRECT, 0);

	return (rctBounding.bottom - rctBounding.top);
}

// 3D text helper
// TODO: Check whether this is correct...
//		 Seems to use hardcoded values instead of from the font structure
HRESULT CreateD3DXTextMesh( int iFont, LPD3DXMESH* ppMesh, TCHAR* pString, FLOAT nSize, int nPrecision, int nAlign)
{
	// Valid font ID
	if ((iFont < 1) || (iFont > g_MaxFonts))
		return 0;
	else
		iFont--;


    HRESULT hr;
    ID3DXMesh *pMeshNew = NULL;
    HDC hdc = CreateCompatibleDC( NULL );
    HFONT hFont;
    HFONT hFontOld;
    FLOAT fPrecision;
    D3DXVECTOR3 minBounds,maxBounds,sum;
    BYTE* pVertices=NULL;
    DWORD nVertices;
    DWORD nVertSize;
    ID3DXMesh *pMesh=NULL;

    switch (nPrecision)
    {
        case 0:
        {
            fPrecision=0.1f;
            break;
        }
        case 1:
        {
            fPrecision=0.01f;
            break;
        }
        case 2:
        {
            fPrecision=0.001f;
            break;
        }
    }
    hFont = CreateFont(15, 0, 0, 0, fiFont[iFont].bBold, fiFont[iFont].bItalic, FALSE, FALSE, fiFont[iFont].dwCharSet,
                       OUT_TT_ONLY_PRECIS, CLIP_DEFAULT_PRECIS, ANTIALIASED_QUALITY, DEFAULT_PITCH | FF_DONTCARE, fiFont[iFont].sFontname);

    hFontOld = (HFONT)SelectObject(hdc, hFont);

	if (fiFont[iFont-1].iCodePage==CP_ACP)
		hr = D3DXCreateText( g_pD3DDevice, hdc, pString, fPrecision, nSize, &pMeshNew, NULL, NULL);
	else
	{
		int lenA = lstrlenA(pString);
		int lenW;
		BSTR unicodestr;

		lenW = ::MultiByteToWideChar(fiFont[iFont].iCodePage, 0, pString, lenA, 0, 0);
		unicodestr = ::SysAllocStringLen(0, lenW);
		::MultiByteToWideChar(fiFont[iFont].iCodePage, 0, pString, lenA, unicodestr, lenW);
		hr = D3DXCreateTextW( g_pD3DDevice, hdc, unicodestr, fPrecision, nSize, &pMeshNew, NULL, NULL);
		::SysFreeString(unicodestr);
	}

    if (FAILED(hr))
        return FALSE;

    hr=pMeshNew->LockVertexBuffer(D3DUSAGE_DYNAMIC, (LPVOID*)&pVertices);
    if (FAILED(hr))
        return FALSE;

    D3DXComputeBoundingBox((D3DXVECTOR3*)pVertices, pMeshNew->GetNumVertices(), D3DXGetFVFVertexSize(pMeshNew->GetFVF()), &minBounds, &maxBounds);

    maxBounds.x=maxBounds.x*1;
    maxBounds.y=maxBounds.y*1;

    nVertices=pMeshNew->GetNumVertices();
    nVertSize=D3DXGetFVFVertexSize(pMeshNew->GetFVF());

    if ( nAlign<0 || nAlign>3)
        nAlign=1;

    switch (nAlign)
    {
        case 0:
        {
            for(DWORD i=0; i < nVertices; i++)
            {
                D3DXVECTOR3 p = *(D3DXVECTOR3*)(pVertices + i*nVertSize);
                p.x=(p.x*1)-minBounds.x;
                p.y=(p.y*1)-(maxBounds.y-minBounds.y)/2;
                p.z=p.z+(maxBounds.z-minBounds.z)/2;
                memcpy(pVertices + i*nVertSize,&p,sizeof(p));
            }
            break;
        }
        case 1:
        {
            for(DWORD i=0; i < nVertices; i++)
            {
                D3DXVECTOR3 p = *(D3DXVECTOR3*)(pVertices + i*nVertSize);
                p.x=(p.x*1)-minBounds.x-(maxBounds.x-minBounds.x)/2;
                p.y=(p.y*1)-(maxBounds.y-minBounds.y)/2;
                p.z=p.z+(maxBounds.z-minBounds.z)/2;
                memcpy(pVertices + i*nVertSize,&p,sizeof(p));
            }
            break;
        }
        case 2:
        {
            for(DWORD i=0; i < nVertices; i++)
            {
                D3DXVECTOR3 p = *(D3DXVECTOR3*)(pVertices + i*nVertSize);
                p.x=(p.x*1)-maxBounds.x;
                p.y=(p.y*1)-(maxBounds.y-minBounds.y)/2;
                p.z=p.z+(maxBounds.z-minBounds.z)/2;
                memcpy(pVertices + i*nVertSize,&p,sizeof(p));
            }
            break;
        }
        case 3:
        {
            for(DWORD i=0; i < nVertices; i++)
            {
                D3DXVECTOR3 p = *(D3DXVECTOR3*)(pVertices + i*nVertSize);
                p.x=(p.x*1)-minBounds.x;
                p.z=p.z+(maxBounds.z-minBounds.z)/2;
                memcpy(pVertices + i*nVertSize,&p,sizeof(p));
            }
            break;
        }
    }

    pMeshNew->UnlockVertexBuffer();


    SelectObject(hdc, hFontOld);
    DeleteObject( hFont );
    DeleteDC( hdc );

    hr=pMeshNew->CloneMeshFVF(D3DXMESH_MANAGED, //Options, see the SDK docs
                              D3DFVF_DBPMESH,         //The FVF we want our new mesh to have
                              g_pD3DDevice,//Our D3D Device
                              &pMesh);

    if( SUCCEEDED( hr ) )
    {
        *ppMesh = pMesh;
        pMeshNew->Release();
    }

    return TRUE;
}

// 3D text object
void Text3D(int iObjectID, int iFont, LPTSTR pString, float fDepth, int iPrecision, int iAlign)
{

    HRESULT HR = CreateD3DXTextMesh(iFont, &g_pMesh3D, pString, fDepth, iPrecision, iAlign);

	if (!HR)
		return;

    MakeObjectFromDXMesh(iObjectID, g_pMesh3D);
}